<?php

// Load in essential map variables from the database.
$geo_comment_default_map_zoom = variable_get('geo_comment_default_map_zoom', 0);
$geo_comment_min_map_zoom = variable_get('geo_comment_min_map_zoom', 'none');
$geo_comment_max_map_zoom = variable_get('geo_comment_max_map_zoom', 'none');

$geo_comment_map_apply_bounds = false;
$geo_comment_map_min_lat = variable_get('geo_comment_map_min_lat', '');
$geo_comment_map_min_lng = variable_get('geo_comment_map_min_lng', '');
$geo_comment_map_max_lat = variable_get('geo_comment_map_max_lat', '');
$geo_comment_map_max_lng = variable_get('geo_comment_map_max_lng', '');
if ($geo_comment_map_min_lat != '' && $geo_comment_map_min_lng != '' && $geo_comment_map_max_lat != '' && $geo_comment_map_max_lng != '') {
  $geo_comment_map_apply_bounds = true;
}

$geo_comment_address_search_suffix = variable_get('geo_comment_address_search_suffix', '');

// See if we need to load the Google Maps API for this page.
global $geo_comments_included_maps_api;
if (!$geo_comments_included_maps_api) {
  drupal_set_html_head('<script type="text/javascript" src="http://maps.googleapis.com/maps/api/js?sensor=false"></script>');
  $geo_comments_included_maps_api = true;
}

?>
<script type="text/javascript">
<!--
// Wrap the Google Map initialization code in a function.
function initializeGoogleMap() {
  /**
   * Variable definitions.
   */
  var defaultMapZoomLevel = <?php echo check_plain($geo_comment_default_map_zoom); ?>,
      minMapZoomLevel = <?php echo ($geo_comment_min_map_zoom == 'none') ? 'null' : check_plain($geo_comment_min_map_zoom); ?>,
      maxMapZoomLevel = <?php echo ($geo_comment_max_map_zoom == 'none') ? 'null' : check_plain($geo_comment_max_map_zoom); ?>,
      boundsApply = <?php echo $geo_comment_map_apply_bounds ? 'true' : 'false'; ?>,
      boundsMinLat = <?php echo $geo_comment_map_min_lat ? check_plain($geo_comment_map_min_lat) : "''"; ?>,
      boundsMaxLat = <?php echo $geo_comment_map_max_lat ? check_plain($geo_comment_map_max_lat) : "''"; ?>,
      boundsMinLng = <?php echo $geo_comment_map_min_lng ? check_plain($geo_comment_map_min_lng) : "''"; ?>,
      boundsMaxLng = <?php echo $geo_comment_map_max_lng ? check_plain($geo_comment_map_max_lng) : "''"; ?>,
      addressSearchSuffix = <?php echo $geo_comment_address_search_suffix ? "'" . check_plain($geo_comment_address_search_suffix) . "'" : "''"; ?>;

  /**
   * Map-related objects.
   */
  // Setup the basic map options.
  var mapOptions = {
    zoom: defaultMapZoomLevel,
    center: new google.maps.LatLng($('#commentLatitude').val(), $('#commentLongitude').val()),
    mapTypeId: google.maps.MapTypeId.ROADMAP
  }
  // Create the Google Map object.
  var map = new google.maps.Map(document.getElementById('google_map_canvas'), mapOptions);
  // Create a Google Maps Geocoder object for running searches.
  var geocoder = new google.maps.Geocoder();
  // Create a marker that the user can use to identify their location.
  var marker = new google.maps.Marker({
    draggable: true,
    position: map.getCenter(),
    map: map,
    title: 'Click and drag me to the desired center-point of your map',
    visible: ($('#toggleMapMarker').is('input[type=checkbox]') && $('#toggleMapMarker').is(":checked")) || ($('#toggleMapMarker').is('input[type=hidden]') && $('#toggleMapMarker').val() == 1) ? true : false
  });
  // Create a button to help the user find the location marker.
  var findPinButton = document.createElement('div');
  findPinButton.style.background = 'none repeat scroll left top white';
  findPinButton.style.border = '1px solid rgb(113, 123, 135)';
  findPinButton.style.boxShadow = '0 2px 4px rgba(0, 0, 0, 0.4)';
  findPinButton.style.color = 'black';
  findPinButton.style.cursor = 'pointer';
  findPinButton.style.direction = 'ltr';
  findPinButton.style.fontFamily = 'Arial,sans-serif';
  findPinButton.style.fontSize = '13px';
  findPinButton.style.fontWeight = 'normal';
  findPinButton.style.margin = '5px 0 5px 5px';
  findPinButton.style.overflow = 'hidden';
  findPinButton.style.padding = '1px 6px';
  findPinButton.style.textAlign = 'center';
  findPinButton.style.display = ($('#toggleMapMarker').is('input[type=checkbox]') && $('#toggleMapMarker').is(":checked")) || ($('#toggleMapMarker').is('input[type=hidden]') && $('#toggleMapMarker').val() == 1) ? 'block' : 'none';
  findPinButton.innerHTML = '<?php echo check_plain(t('Find Marker')); ?>';
  findPinButton.title = '<?php echo check_plain(t('Centre the map on the marker')); ?>';
  findPinButton.index = 1;
  findPinButton.id = 'findPinButton';
  map.controls[google.maps.ControlPosition.TOP_RIGHT].push(findPinButton);

  /**
   * Interface events.
   */
  // Listen for the user clicking on the 'Find Marker' button.
  google.maps.event.addDomListener(findPinButton, 'click', function (event) {
    map.setCenter(marker.position);
  });
  // Listen for the user toggling the map marker on and off.
  google.maps.event.addDomListener(document.getElementById('toggleMapMarker'), 'change', function (event) {
    var fieldLat = document.getElementById('commentLatitude');
    var fieldLng = document.getElementById('commentLongitude');
    if (event.target.checked) {
      marker.setVisible(true);
      document.getElementById('findPinButton').style.display = 'block';
    }
    else {
      marker.setVisible(false);
      document.getElementById('findPinButton').style.display = 'none';
    }
  });
  // Reposition the marker when the user clicks somewhere on the map.
  google.maps.event.addDomListener(map, 'click', function(event) {
    marker.setPosition(event.latLng);
    document.getElementById('commentLatitude').value = event.latLng.lat().toFixed(5);
    document.getElementById('commentLongitude').value = event.latLng.lng().toFixed(5);
    if (boundsApply) {
      // Check that the marker lies within the map bounds.
      checkPositionBounds();
    }
  });
  // Listen for 'drag end' events on the marker and record the dropped location.
  google.maps.event.addListener(marker, 'dragend', function (event) {
    document.getElementById('commentLatitude').value = event.latLng.lat().toFixed(5);
    document.getElementById('commentLongitude').value = event.latLng.lng().toFixed(5);
    if (boundsApply) {
      // Check that the marker lies within the map bounds.
      checkPositionBounds();
    }
  });
  // Listen for 'zoom changed' events and, if necessary, constrain the zoom level to the levels specified in the Geo-comments admin.
  google.maps.event.addListener(map, 'zoom_changed', function() {
    if (minMapZoomLevel != null && map.getZoom() < minMapZoomLevel) {
      map.setZoom(minMapZoomLevel);
    }
    if (maxMapZoomLevel != null && map.getZoom() > maxMapZoomLevel) {
      map.setZoom(maxMapZoomLevel);
    }
  });
  // Listen for 'center changed' events and reset the map if it pans out of the allowed bounds.
  google.maps.event.addListener(map, 'center_changed', function() {
    if (boundsApply) {
      // Check that the center of the map hasn't moved outside of the bounds.
      checkMapBounds();
    }
  });
  // Listen for the enter key being pressed on the map search field and run the search.
  $('#geo_comments_map_search').keypress(function(event) {
    if ( event.which == 13 ) {
      event.preventDefault();
      mapSearch($(this).val());
    }
  });

  /**
   * Helper functions.
   */
  // Restrict panning of the map to within the specified bounds.
  function checkMapBounds() {
    allowedBounds = getAllowedBounds();
    if(!allowedBounds.contains(map.getCenter())) {
      var C = map.getCenter();
      var X = C.lng();
      var Y = C.lat();

      var AmaxX = allowedBounds.getNorthEast().lng();
      var AmaxY = allowedBounds.getNorthEast().lat();
      var AminX = allowedBounds.getSouthWest().lng();
      var AminY = allowedBounds.getSouthWest().lat();

      if (X < AminX) {X = AminX;}
      if (X > AmaxX) {X = AmaxX;}
      if (Y < AminY) {Y = AminY;}
      if (Y > AmaxY) {Y = AmaxY;}

      map.setCenter(new google.maps.LatLng(Y,X));
    }
  }
  // Confirm that the given position falls within the allowed bounds and sets
  // the map marker to the nearest available position if not.
  function checkPositionBounds() {
    allowedBounds = getAllowedBounds();
    position = marker.getPosition();
    if(!allowedBounds.contains(position)) {
      var X = position.lng();
      var Y = position.lat();

      var AmaxX = allowedBounds.getNorthEast().lng();
      var AmaxY = allowedBounds.getNorthEast().lat();
      var AminX = allowedBounds.getSouthWest().lng();
      var AminY = allowedBounds.getSouthWest().lat();

      if (X < AminX) {X = AminX;}
      if (X > AmaxX) {X = AmaxX;}
      if (Y < AminY) {Y = AminY;}
      if (Y > AmaxY) {Y = AmaxY;}

      marker.setPosition(new google.maps.LatLng(Y,X));
      document.getElementById('commentLatitude').value = Y.toFixed(5);
      document.getElementById('commentLongitude').value = X.toFixed(5);
    }
  }
  // Perform a text-search on the Google Map to center it on a particular location.
  function mapSearch(search) {
    // If bounds are set, send them to the geocoder. Otherwise, send the viewport bounds.
    allowedBounds = getAllowedBounds();
    var bounds = boundsApply ? allowedBounds : map.getBounds();
    geocoder.geocode(
      {
        'address': search + ((addressSearchSuffix != '') ? ' ' + addressSearchSuffix : ''),
        'bounds': bounds
      }, 
      function(results, status) { 
        var selectBox = $('#geo_comments_map_search_results');
        if (status == google.maps.GeocoderStatus.OK) {
          var location = results[0].geometry.location;
          if(boundsApply && !allowedBounds.contains(location)) {
            // Location not found within the given bounds.
            selectBox.hide();
          }
          else {
            // Home the map in on the first location found.
            map.setCenter(location);
            // Empty the selection box.
            $('option', selectBox).remove();
            // Don't bother showing the search result selection box if there's only one match.
            if (results.length <= 1) {
              selectBox.hide();
            }
            else {
              // Get the set of options already belonging to the selection box.
              if (selectBox.prop) {
                var options = selectBox.prop('options');
              }
              else {
                var options = selectBox.attr('options');
              }
              // Add all the results to the selection box.
              $.each(results, function(index, loc) {
                // Check that the result location is within our bounds.
                if (allowedBounds.contains(loc.geometry.location)) {
                  options[options.length] = new Option(loc.formatted_address, loc.geometry.location.lat() + '_' + loc.geometry.location.lng());
                }
              });
              selectBox.show();
            }
          }
        } 
        else {
          // Location search came up with nothing.
          selectBox.hide();
        }
      }
    );
  };
  // Re-center the map when the user clicks on a result in the map search
  // results box.
  $('#geo_comments_map_search_results').change(function(){
    var searchBox = $(this);
    var latLng = searchBox.val().split('_');
    var location = new google.maps.LatLng(latLng[0], latLng[1]);
    map.setCenter(location);
  });
  // Convenience function that creates a Google LatLngBounds object defining the
  // rectangle that contains the map's allowable area.
  function getAllowedBounds() {
    return new google.maps.LatLngBounds(
      new google.maps.LatLng(boundsMinLat, boundsMinLng),
      new google.maps.LatLng(boundsMaxLat, boundsMaxLng));
  }
}

// Trigger loading of the Google Map javascript on doc ready.
$(function(){
  initializeGoogleMap();
});
-->
</script>

<div class="form-item">
  <label><?php echo t('Tag your location:'); ?><?php if ($element['#custom_required']) { ?> <span title="<?php echo t('This field is required.'); ?>" class="form-required">*</span><?php } ?></label>
  <div id="google_map_canvas" style="width: 400px; height: 300px;"></div>
  <div>
    <label for="geo_comments_map_search">Map Search:</label> <input type="text" id="geo_comments_map_search" />
  </div>
  <div>
    <select id="geo_comments_map_search_results" size="5" style="display: none;"></select>
  </div>
</div>
