<?php

// Load in essential map variables from the database.
$geo_comment_default_map_zoom = variable_get('geo_comment_default_map_zoom', 0);
$geo_comment_min_map_zoom = variable_get('geo_comment_min_map_zoom', 'none');
$geo_comment_max_map_zoom = variable_get('geo_comment_max_map_zoom', 'none');

$geo_comment_default_map_center_lat = variable_get('geo_comment_default_map_center_lat', 51.5);
$geo_comment_default_map_center_lng = variable_get('geo_comment_default_map_center_lng', 0);

$geo_comment_map_apply_bounds = false;
$geo_comment_map_min_lat = variable_get('geo_comment_map_min_lat', '');
$geo_comment_map_min_lng = variable_get('geo_comment_map_min_lng', '');
$geo_comment_map_max_lat = variable_get('geo_comment_map_max_lat', '');
$geo_comment_map_max_lng = variable_get('geo_comment_map_max_lng', '');
if ($geo_comment_map_min_lat != '' && $geo_comment_map_min_lng != '' && $geo_comment_map_max_lat != '' && $geo_comment_map_max_lng != '') {
  $geo_comment_map_apply_bounds = true;
}

// Load a set of comments for this node, including their lat/lng co-ordinates.
$geo_comments = geo_comments_load_comments($node->nid);

  // See if we need to load the Google Maps API for this page.
  global $geo_comments_included_maps_api;
  if (!$geo_comments_included_maps_api) {
    drupal_set_html_head('<script type="text/javascript" src="http://maps.googleapis.com/maps/api/js?sensor=false"></script>');
    $geo_comments_included_maps_api = true;
  }

?>

<script type="text/javascript">
<!--
var map = null;
var infoWindow = null;
var comments = [];

// Wrap the Google Map initialization code in a function.
function initializeGoogleMap() {
  /**
   * Variable definitions.
   */
  var defaultMapZoomLevel = <?php echo check_plain($geo_comment_default_map_zoom); ?>,
      minMapZoomLevel = <?php echo ($geo_comment_min_map_zoom == 'none') ? 'null' : check_plain($geo_comment_min_map_zoom); ?>,
      maxMapZoomLevel = <?php echo ($geo_comment_max_map_zoom == 'none') ? 'null' : check_plain($geo_comment_max_map_zoom); ?>,
      boundsApply = <?php echo $geo_comment_map_apply_bounds ? 'true' : 'false'; ?>,
      boundsMinLat = <?php echo $geo_comment_map_min_lat ? check_plain($geo_comment_map_min_lat) : "''"; ?>,
      boundsMaxLat = <?php echo $geo_comment_map_max_lat ? check_plain($geo_comment_map_max_lat) : "''"; ?>,
      boundsMinLng = <?php echo $geo_comment_map_min_lng ? check_plain($geo_comment_map_min_lng) : "''"; ?>,
      boundsMaxLng = <?php echo $geo_comment_map_max_lng ? check_plain($geo_comment_map_max_lng) : "''"; ?>,
      mapCenterLat = <?php echo check_plain($geo_comment_default_map_center_lat); ?>,
      mapCenterLng = <?php echo check_plain($geo_comment_default_map_center_lng); ?>;

  /**
   * Map-related objects.
   */
  // Setup the basic map options.
  var mapOptions = {
    zoom: defaultMapZoomLevel,
    center: new google.maps.LatLng(mapCenterLat, mapCenterLng),
    mapTypeId: google.maps.MapTypeId.ROADMAP
  }
  // Create the Google Map object.
  map = new google.maps.Map(document.getElementById('google_comment_map_canvas'), mapOptions);

  /**
   * Interface events.
   */
  // Listen for 'zoom changed' events and, if necessary, constrain the zoom level to the levels specified in the Geo-comments admin.
  google.maps.event.addListener(map, 'zoom_changed', function() {
    constrainMapZoom();
  });
  function constrainMapZoom() {
    if (minMapZoomLevel != null && map.getZoom() < minMapZoomLevel) {
      map.setZoom(minMapZoomLevel);
    }
    if (maxMapZoomLevel != null && map.getZoom() > maxMapZoomLevel) {
      map.setZoom(maxMapZoomLevel);
    }
  }

  /**
   * Helper functions.
   */
  // Restrict panning of the map to within the specified bounds.
  function checkMapBounds(allowedBounds) { 
    if(!allowedBounds.contains(map.getCenter())) {
      var C = map.getCenter();
      var X = C.lng();
      var Y = C.lat();

      var AmaxX = allowedBounds.getNorthEast().lng();
      var AmaxY = allowedBounds.getNorthEast().lat();
      var AminX = allowedBounds.getSouthWest().lng();
      var AminY = allowedBounds.getSouthWest().lat();

      if (X < AminX) {X = AminX;}
      if (X > AmaxX) {X = AmaxX;}
      if (Y < AminY) {Y = AminY;}
      if (Y > AmaxY) {Y = AmaxY;}

      map.setCenter(new google.maps.LatLng(Y,X));
    }
  }

  // Create an object for storing the maximum extents of the various map markers.
  var markersBounds = new google.maps.LatLngBounds();
  // Create the InfoWindow object for displaying the comments on the map.
  infoWindow = new google.maps.InfoWindow();

  // This helper function lets us use jQuery to select an element with a
  // forward-slash in its ID.
  function safeThread(thread) {
    return thread.replace('/', '\\\\/').replace('.', '\\\\.');
  }

    // Create a new marker on the Google map relating to the given comment.
function newCommentMarker(comment) {
    var commentDiv = document.getElementById('geo_comment_content_' + comment.thread);
    if (commentDiv) {
      // Create the new marker object.
      marker = new google.maps.Marker({
        position: comment.pos,
        title: comment.title,
        animation: google.maps.Animation.DROP,
        map: map
      });
      // Increase the markersBounds object, if necessary, to accomodate this
      // comment's location.
      markersBounds.extend(comment.pos);
      // Add a listener for when the user clicks on a marker, so we can show the
      // comment in the info window on the map.
      google.maps.event.addListener(marker, 'click', function() {
        populateGeoCommentInfoWindow(comment);
      });
      // Save a copy of the marker so that we can navigate to it later.
      comment.marker = marker;
    }
  }

<?php
// Output all the details we need for each Geo-comment in the javascript.
$comment_count = 0;
foreach ($geo_comments as $comment) {
  if (!empty($comment->latitude) && !empty($comment->longitude)) {
?>
  comments[comments.length] = {
    cid: <?php echo $comment->cid; ?>,
    thread: '<?php echo $comment->thread; ?>',
    pos: new google.maps.LatLng(<?php echo $comment->latitude . ', ' . $comment->longitude; ?>),
    title: '<?php echo htmlentities($comment->subject, ENT_QUOTES); ?>',
    frtt: '<?php echo array_key_exists($comment->thread_links['first_reply_to_this'], $geo_comments) ? $comment->thread_links['first_reply_to_this'] : ''; ?>',
    nrtp: '<?php echo array_key_exists($comment->thread_links['next_reply_to_parent'], $geo_comments) ? $comment->thread_links['next_reply_to_parent'] : ''; ?>',
    prc: '<?php echo array_key_exists($comment->thread_links['previous_comment'], $geo_comments) ? $comment->thread_links['previous_comment'] : ''; ?>',
    pac: '<?php echo array_key_exists($comment->thread_links['parent_comment'], $geo_comments) ? $comment->thread_links['parent_comment'] : ''; ?>'
  };
<?php
    $comment_count++;
  }
}
?>
  // Create the new map marker for each comment.
  for (var i = 0; i < comments.length; i++) {
    newCommentMarker(comments[i]);
  }
  if (comments.length > 0) {
    // Expand the map's viewport to fit all the markers.
    map.fitBounds(markersBounds);
    // Check that the viewport is within the min/max zoom levels.
    constrainMapZoom();
  }
}

// Trigger loading of the Google Map javascript on doc ready.
$(function(){
  initializeGoogleMap();
});

// Tells the Google maps to open the specified Geo-comment info window.
function openGeoCommentInfoWindow(index) {
  populateGeoCommentInfoWindow(comments[index]);
  infoWindow.open(map, comments[index].marker);
  return false;
}

// Adds the specified comment's content to the Geo-comment info window.
function populateGeoCommentInfoWindow(comment) {
  var commentDiv = document.getElementById('geo_comment_content_' + comment.thread);
  if (commentDiv) {
    var infoWindowContent = commentDiv.innerHTML;
    for (var i = 0; i < comments.length; i++) {
      if (comments[i].thread == comment.frtt) {
        infoWindowContent += '<div class="geo_comment_info_link"><a id="geo_link_' + comment.cid + '_to_' + comment.frtt + '" href="#" onclick="return openGeoCommentInfoWindow(' + i + ');">Read reply to this (' + comment.frtt + ')</a></div>';
      }
      if (comments[i].thread == comment.nrtp) {
        infoWindowContent += '<div class="geo_comment_info_link"><a id="geo_link_' + comment.cid + '_to_' + comment.nrtp + '" href="#" onclick="return openGeoCommentInfoWindow(' + i + ');">Next reply (' + comment.nrtp + ')</a></div>';
      }
      if (comments[i].thread == comment.prc) {
        infoWindowContent += '<div class="geo_comment_info_link"><a id="geo_link_' + comment.cid + '_to_' + comment.prc + '" href="#" onclick="return openGeoCommentInfoWindow(' + i + ');">Previous reply (' + comment.prc + ')</a></div>';
      }
      if (comments[i].thread == comment.pac) {
        infoWindowContent += '<div class="geo_comment_info_link"><a id="geo_link_' + comment.cid + '_to_' + comment.pac + '" href="#" onclick="return openGeoCommentInfoWindow(' + i + ');">This is a reply to (' + comment.pac + ')</a></div>';
      }
    }
    infoWindow.setContent(infoWindowContent);
    infoWindow.open(map, comment.marker);
  }
}

-->
</script>

<h3>Geo-comments</h3>
<p class="description">Click a pin to read the comment from the user in that location.</p>

<div id="google_comment_map_canvas" style="width: 600px; height: 350px;"></div>

<?php
// Output the comments with standard themed formatting.
echo $content;
?>

<?php
// Output the comment contents in hidden DIVs, ready for loading into the map.
$comment_count = 0;
foreach ($geo_comments as $comment) {
  if (!empty($comment->latitude) && !empty($comment->longitude)) {
?>
<div id="geo_comment_content_<?php echo $comment->thread; ?>" style="width: 300px; display: none;">
  <div class="geo_comment_content_posted">On <span><?php echo date("D, jS F Y", $comment->timestamp); ?></span>, <strong><?php echo htmlentities($comment->name); ?></strong> commented:</div>
  <div class="geo_comment_content_content"><?php echo $comment->comment; ?></div>
</div>
<?php
    $comment_count++;
  }
}
?>
